#include <windows.h>
#include <stdio.h>
#include <imagehlp.h>

#pragma comment(lib, "imagehlp.lib")


//Validates a PE file by checking the DOS and NT signatures.
bool IsValidPeFile(PVOID BaseAddress, DWORD FileLength)
{
	PIMAGE_DOS_HEADER DosHeader = (PIMAGE_DOS_HEADER)BaseAddress;
	PIMAGE_NT_HEADERS NtHeaders;

	if( DosHeader->e_magic != IMAGE_DOS_SIGNATURE )
		return false;

	NtHeaders = (PIMAGE_NT_HEADERS)((PBYTE)DosHeader + DosHeader->e_lfanew);
	
	//Do the NT headers lie within our image?
	if( (ULONG_PTR)NtHeaders + sizeof(IMAGE_NT_HEADERS) > (ULONG_PTR)BaseAddress + FileLength )
		return false;

	if( NtHeaders->Signature != IMAGE_NT_SIGNATURE )
		return false;

	return true;
}

void StripDebugInfo(PVOID BaseAddress, DWORD FileLength)
{
	PIMAGE_NT_HEADERS NtHeaders;
	PIMAGE_SECTION_HEADER DebugSectionHeader;
	PIMAGE_DEBUG_DIRECTORY DebugDirectory;
	PIMAGE_DATA_DIRECTORY DirectoryEntry;
	PVOID DebugData;
	DWORD Size;

	printf("Zeroing debug directory...");

	NtHeaders = (PIMAGE_NT_HEADERS)((PBYTE)BaseAddress + ((PIMAGE_DOS_HEADER)BaseAddress)->e_lfanew);

	DebugData = ImageDirectoryEntryToDataEx(BaseAddress, FALSE, IMAGE_DIRECTORY_ENTRY_DEBUG, 
					&Size, &DebugSectionHeader);

	DebugDirectory = (PIMAGE_DEBUG_DIRECTORY)DebugData;

	//If no directory, exit.
	if( !DebugDirectory || !DebugDirectory->PointerToRawData || 0 == DebugDirectory->SizeOfData )
	{
		printf(" not found!\n");
		return;
	}

	//Zero raw data.
	memset((PVOID)(DebugDirectory->PointerToRawData + (ULONG_PTR)BaseAddress), 0, DebugDirectory->SizeOfData);

	//Zero the directory.
	memset(DebugData, 0, Size);

	//Finally, find the actual directory entry and zero it.
	DirectoryEntry = &NtHeaders->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_DEBUG];
	memset(DirectoryEntry, 0, sizeof(IMAGE_DATA_DIRECTORY));

	printf(" done!\n");
}

void CleanRichInfo(PVOID BaseAddress, DWORD FileLength)
{
	PDWORD CurrentValue;
	PDWORD RichHeaderStart = 0, RichHeaderEnd = 0;
	DWORD XorKey, i;

	printf("Stripping 'Rich' header...");

	//Search forward looking for the "Rich" signature.
	CurrentValue = (PDWORD)BaseAddress;
	for( i = 0; i < 0x100; i += 4, CurrentValue++ )
	{
		if( *CurrentValue == 0x68636952 )
		{
			//Got it!
			RichHeaderEnd = ++CurrentValue;
			break;
		}
	}

	//If we didn't find it, exit.
	if( !RichHeaderEnd )
	{
		printf(" not found!\n");
		return;
	}

	//Get XOR key.
	XorKey = *RichHeaderEnd;

	//Go past the end, so we zero this DWORD below.
	RichHeaderEnd++;

	//Loop backwards, XOR-ing until we find the block start.
	for( ; (ULONG_PTR)CurrentValue >= (ULONG_PTR)BaseAddress; CurrentValue-- )
	{
		if( ((*CurrentValue) ^ XorKey) == 'SnaD' )		//Compare for 'DanS', which is the start.
		{
			//Got the start!
			RichHeaderStart = CurrentValue;
			break;
		}
	}

	if( !RichHeaderStart )
	{
		printf(" not found!\n");
		return;
	}

	//Zero it out...
	memset(RichHeaderStart, 0, (ULONG_PTR)RichHeaderEnd - (ULONG_PTR)RichHeaderStart);

	printf(" done!\n");
}

void CleanMiscInfo(PVOID BaseAddress, DWORD FileLength)
{
	PIMAGE_NT_HEADERS NtHeaders;

	//Get NT headers.
	NtHeaders = (PIMAGE_NT_HEADERS)((PBYTE)BaseAddress + ((PIMAGE_DOS_HEADER)BaseAddress)->e_lfanew);
	
	//Remove linker info.
	NtHeaders->OptionalHeader.MajorLinkerVersion = 0;
	NtHeaders->OptionalHeader.MinorLinkerVersion = 0;
}

int wmain(int argc, wchar_t * argv[])
{
	HANDLE hFile;
	HANDLE hMapping;
	LPVOID BaseAddress;
	DWORD FileLength;
	DWORD HeaderSum, CheckSum;

	//If we have arguments...
	if( argc <= 1 )
	{
		printf("Usage: StripPeFile filename.exe\n\n");
		return ERROR_FILE_NOT_FOUND;
	} else 
	{
		printf("Stripping file '%ws'...\n", argv[1]);
	}

	//Open the file.
	hFile = CreateFileW(argv[1],
			  GENERIC_ALL,
			  FILE_SHARE_READ | FILE_SHARE_WRITE,
			  NULL,
			  OPEN_EXISTING,
			  FILE_ATTRIBUTE_NORMAL,
			  0);

	if( INVALID_HANDLE_VALUE == hFile )
	{
		return GetLastError();
	}

	//Map a view of the file.
	hMapping = CreateFileMapping(hFile,
				   NULL,
				   PAGE_READWRITE,
				   0,
				   0,
				   NULL);
	if( 0 == hMapping )
	{
		CloseHandle(hFile);
		return GetLastError();
	}

	//Map a view for editing.
	BaseAddress = MapViewOfFile(hMapping,
					  FILE_MAP_READ | FILE_MAP_WRITE,
					  0,
					  0,
					  0);

	if( NULL == BaseAddress )
	{
		CloseHandle(hMapping);
		CloseHandle(hFile);
		return GetLastError();
	}

	//Get the size of the file.
	FileLength = GetFileSize(hFile, NULL);

	//Validate PE file.
	if( IsValidPeFile(BaseAddress, FileLength) )
	{
		//Strip debug information.
		StripDebugInfo(BaseAddress, FileLength);

		//Clean RICH header.
		CleanRichInfo(BaseAddress, FileLength);

		//Clean miscellaneous information.
		CleanMiscInfo(BaseAddress, FileLength);

		//Update the checksum.
		PIMAGE_NT_HEADERS ImageHeaders = CheckSumMappedFile(BaseAddress, FileLength, &HeaderSum, &CheckSum);

		//Save the checksum.
		printf("Updating checksum...\r\n");
		if( ImageHeaders )
			ImageHeaders->OptionalHeader.CheckSum = CheckSum;
	} else 
	{
		printf("Invalid PE file!\n");
	}

	//Close down...
	UnmapViewOfFile(BaseAddress);
	CloseHandle(hMapping);
	CloseHandle(hFile);

	printf("Done!\r\n");
	//system("pause");
	return 0;
}
